# MIT License
#
# Copyright (c) 2020, Niklas Rosenstein
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# this software and associated documentation files (the "Software"), to deal in
# Software without restriction, including without limitation the rights to use,
# modify, merge, publish, distribute, sublicense, and/or sell copies of the
# and to permit persons to whom the Software is furnished to do so, subject to
# following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
# USE OR OTHER DEALINGS IN THE SOFTWARE.

"""
cube.daemon.api
===============

Provides the cube-daemon JSON REST API.
"""


import flask

from cube import spec
from cube.daemon.config import ApplicationConfig
from nr.databind.core import ObjectMapper
from nr.databind.json import JsonModule
from nr.databind.rest import MimeTypeMapper
from nr.databind.rest.flask import bind_resource
from nr.proxy import Proxy

app = flask.Flask(__name__)

# Global proxies for app configuration values.
spec_context: spec.Context = Proxy(lambda: app.config['CUBE_DAEMON_SPEC_CONTEXT'])
config: ApplicationConfig = Proxy(lambda: app.config['CUBE_DAEMON_CONFIG'])
mapper: ObjectMapper = Proxy(lambda: app.config.get('CUBE_DAEMON_OBJECT_MAPPER'))
mime_mapper: MimeTypeMapper = MimeTypeMapper.json(mapper)

from cube.daemon.api import services
bind_resource(app, '/api/config', services.ConfigServiceImpl(), mapper=mime_mapper)
bind_resource(app, '/api/products', services.ProductsServiceImpl(), mapper=mime_mapper)
bind_resource(app, '/api/services', services.ServicesServiceImpl(), mapper=mime_mapper)
